/*===========================================================================*/
/*                                                                           */
/* File    : BUTTON.C                                                        */
/*                                                                           */
/* Purpose : Demonstrates different kinds of buttons available in MEWEL.     */
/*           Also demos the use of owner-drawn pushbuttons.                  */
/*                                                                           */
/* History :                                                                 */
/*                                                                           */
/* (C) Copyright 1989-1992 Marc Adler/Magma Systems     All Rights Reserved  */
/*===========================================================================*/
#include "wmewldem.h"


VOID FAR PASCAL ButtonDemo(void)
{
  FARPROC lpfn;

  /*
    Invoke the Button dialog box
  */
  lpfn = MakeProcInstance((FARPROC) ButtonDialogProc, hAppInstance);
  DialogBox(hAppInstance, "DlgButtonStyles", hMain, lpfn);
  FreeProcInstance(lpfn);
}


int FAR PASCAL ButtonDialogProc(HWND hDlg, WORD msg, WORD wParam, DWORD lParam)
{
  (void) lParam;

  switch (msg)
  {
    case WM_INITDIALOG :
      return TRUE;

    case WM_COMMAND :
      switch (wParam)
      {
        case IDOK :
        case IDCANCEL :
          EndDialog(hDlg, TRUE);
          break;
      }
      return TRUE;


    /*
      Code responsible for drawing the owner-drawn pushbutton
    */
    case WM_DRAWITEM :
    {
      LPDRAWITEMSTRUCT lpDis = (LPDRAWITEMSTRUCT) lParam;
      HDC    hDC = lpDis->hDC;  /* this is a window DC */
      HBRUSH hBrush;
      RECT   r;

      /*
        Draw the button frame
      */
      r = lpDis->rcItem;
      Rectangle(hDC, r.left, r.top, r.right, r.bottom);
      InflateRect(&r, -1, -1);

      /*
        Fill the background with the selected or unselected color
      */
      if (lpDis->itemState & ODS_SELECTED)
        hBrush = CreateSolidBrush(RGB(0x00, 0x00, 0xFF));
      else
        hBrush = CreateSolidBrush(RGB(0x00, 0xFF, 0xFF));
      FillRect(hDC, &r, hBrush);
      DeleteObject(hBrush);

      /*
        Draw the bitmap in the middle
      */
      DrawIcon(hDC,
               (r.right - r.left) / 2 - (GetSystemMetrics(SM_CXICON) / 2),
               (r.bottom - r.top) / 2 - (GetSystemMetrics(SM_CYICON) / 2),
               LoadIcon(NULL, IDI_HAND));

      /*
        If the button has the focus, invert it
      */
      if (lpDis->itemState & ODS_FOCUS)
        InvertRect(hDC, &lpDis->rcItem);

      return TRUE;
    }
  }
  return FALSE;
}

