#include "wmewldem.h"

#define HOURSCALE       65
#define MINUTESCALE     80
#define RHOURSCALE      15
#define RMINUTESCALE    20
#define WHOURSCALE      7
#define WMINUTESCALE    5
#define HHAND           TRUE
#define MHAND           FALSE
#define SECONDSCALE     80
#define MAXBLOBWIDTH    25
#define REPAINT         0
#define HANDPAINT       1

#define OPEN_TLEN       450
#define ICON_TLEN       59000

#define BUFLEN          30

long FAR PASCAL ClockWndProc( HWND, unsigned, WORD, LONG );
static VOID PASCAL CreateTools(void);
static VOID PASCAL DeleteTools(void);
static VOID PASCAL ClockCreate(HWND);
static VOID PASCAL CompClockDim(void);
static BOOL PASCAL ClockInit(HANDLE);
static VOID PASCAL DrawFatHand(HDC, int, HPEN, BOOL);
static VOID PASCAL DrawHand(HDC, int, HPEN, int, int);
static VOID PASCAL DrawFace(HDC);
static VOID PASCAL ClockPaint(HWND, HDC, int);
static VOID PASCAL ClockTimer(HWND, WORD);
static VOID PASCAL ClockSize(HWND, int, int, WORD);


typedef struct {                        /*structure for holding time*/
    int hour;
    int minute;
    int second;
} TIME;

extern void GetTime( TIME * );

HBRUSH hbrBackground;
HBRUSH hbrForeground;
HPEN   hpenForeground;
HPEN   hpenBackground;
BOOL   bFirst = TRUE;
BOOL   bIconic = FALSE;


int TimerID = 1;        /* number used for timer-id */
POINT FAR *lpcirTab;
HANDLE hcirTab;
TIME oTime;
RECT clockRect;
int clockRadius;
POINT clockCenter;
long aspectD;
long aspectN;
int VertRes, HorzRes;


long FAR PASCAL ClockWndProc( hWnd, message, wParam, lParam)
  HWND hWnd;
  unsigned message;
  WORD wParam;
  LONG lParam;
{
  PAINTSTRUCT ps;
  HDC hDC;

  switch (message)
  {
    case WM_SIZE:
      ClockSize( hWnd, LOWORD(lParam), HIWORD(lParam), wParam );
      UpdateWindow( hWnd );
      break;

    case WM_DESTROY:
      KillTimer( hWnd, TimerID );
      DeleteTools();
      break;

    case WM_PAINT:
      InvalidateRect(hWnd, (LPRECT)NULL, TRUE);
      BeginPaint( hWnd, (LPPAINTSTRUCT)&ps );
      ClockPaint( hWnd, ps.hdc, REPAINT );
      EndPaint( hWnd, (LPPAINTSTRUCT)&ps );
      break;

    case WM_TIMECHANGE:
    case WM_TIMER:
      ClockTimer( hWnd, wParam );
      break;

    case WM_SYSCOLORCHANGE:
      DeleteTools();
      CreateTools();
      break;

    case WM_ERASEBKGND:
    {
      RECT rect;
      GetClientRect( hWnd, (LPRECT)&rect );
      SelectObject( (HDC)wParam, hbrBackground );
      FillRect( (HDC)wParam, (LPRECT)&rect, hbrBackground );
      break;
    }

    default:
    defproc:
      return( DefWindowProc(hWnd, message, wParam, lParam) );
  }
  return( 0L );
}


static VOID PASCAL CreateTools(void)
{
  hbrForeground  = CreateSolidBrush(GetSysColor(COLOR_WINDOWTEXT));
  hbrBackground  = CreateSolidBrush(GetSysColor(COLOR_WINDOW));
  hpenForeground = CreatePen(0, 1, GetSysColor(COLOR_WINDOWTEXT));
  hpenBackground = CreatePen(0, 1, GetSysColor(COLOR_WINDOW));
}

static VOID PASCAL DeleteTools(void)
{
  DeleteObject(hbrForeground);
  DeleteObject(hbrBackground);
  DeleteObject(hpenForeground);
  DeleteObject(hpenBackground);
}


static VOID PASCAL ClockCreate(HWND hWnd)
{
  HDC     hDC;
  int     i;
  int     VertSize, HorzSize;
  POINT   FAR *pt;
  RECT    rc;

  /*
    Calculate the aspect ratio of the display
  */
  hDC = GetDC(hWnd);
  VertRes = GetDeviceCaps( hDC, VERTRES );
  HorzRes = GetDeviceCaps( hDC, HORZRES );
  VertSize= GetDeviceCaps( hDC, VERTSIZE );
  HorzSize= GetDeviceCaps( hDC, HORZSIZE );
  ReleaseDC(hWnd, hDC);

  aspectN = ((long)VertRes*100) / (long)VertSize;
  aspectD = ((long)HorzRes*100) / (long)HorzSize;

  CreateTools();

  /* Scale cosines for aspect ratio if this is the first instance */
  if (bFirst)
  {
    lpcirTab = (POINT FAR *) GlobalLock(hcirTab);
    for (i = 0;  i < 60;  i++)
    {
      pt = lpcirTab + i;
      pt->y = ((pt->y) * aspectN) / aspectD;
    }
    GlobalUnlock( hcirTab );
  }

  GetTime((TIME *) &oTime);
}


static VOID PASCAL ClockSize( hWnd, newWidth, newHeight, SizeWord )
  HWND hWnd;
  int newWidth, newHeight;
  WORD  SizeWord;
{
  SetRect( (LPRECT)&(clockRect), 0, 0, newWidth, newHeight );
  CompClockDim();

  if (SizeWord == SIZEICONIC)
  {
    /* Update once every minute in the iconic state */
    KillTimer( hWnd, TimerID );
    SetTimer( hWnd, TimerID, (unsigned) ICON_TLEN, (long)0 );
    bIconic = TRUE;
  }
  else if (bIconic)
  {
    /* Update every second in the opened state.  Ignore tiling */
    KillTimer( hWnd, TimerID );
    SetTimer( hWnd, TimerID, OPEN_TLEN, (long)0 );
    bIconic = FALSE;
  }
}


static VOID PASCAL ClockTimer( hWnd, msg )
  HWND hWnd;                /* A handle to a window data structure  */
  WORD msg;                 /* timer ID */
{
  TIME nTime;
  HDC hDC;

  (void) msg;

  GetTime( (TIME *) &nTime );

  if (nTime.second == oTime.second &&
      nTime.minute == oTime.minute &&
      nTime.hour   == oTime.hour)
    return;

  hDC = GetDC( hWnd );
  ClockPaint( hWnd, hDC, HANDPAINT );
  ReleaseDC( hWnd, hDC );
}


static VOID PASCAL ClockPaint( hWnd, hDC, hint )
  HWND  hWnd;
  HDC   hDC;
  int   hint;
{
  TIME    nTime;

  (void) hWnd;

  lpcirTab = (POINT FAR *)GlobalLock( hcirTab );

  SetBkMode( hDC, TRANSPARENT );
  if (hint == REPAINT)
  {
    FillRect( hDC, (LPRECT)&clockRect, hbrBackground );
    DrawFace( hDC );
    DrawFatHand( hDC, oTime.hour*5+(oTime.minute/12),
                 hpenForeground, HHAND );
    DrawFatHand( hDC, oTime.minute, hpenForeground, MHAND );
    if (!bIconic)
      /* Erase old second hand */
      DrawHand( hDC, oTime.second, hpenBackground, SECONDSCALE, R2_NOT );
  }
  else
  if (hint == HANDPAINT)
  {
    GetTime( (TIME *) &nTime );
    if ( (!bIconic) && nTime.second != oTime.second)
      /* Erase old second hand */
      DrawHand( hDC, oTime.second, hpenBackground, SECONDSCALE, R2_NOT );

    if (nTime.minute != oTime.minute || nTime.hour != oTime.hour)
    {
      if (bIconic) {
        DrawHand( hDC, oTime.minute, hpenBackground, MINUTESCALE, R2_COPYPEN );
        DrawHand( hDC, oTime.hour*5+(oTime.minute/12),
                        hpenBackground, HOURSCALE, R2_COPYPEN );
        DrawHand( hDC, oTime.minute=nTime.minute,
                        hpenForeground, MINUTESCALE, R2_COPYPEN );
        DrawHand( hDC, (oTime.hour=nTime.hour)*5+(oTime.minute/12),
                        hpenForeground, HOURSCALE, R2_COPYPEN );
     }
     else
     {
        DrawFatHand( hDC, oTime.minute,
                           hpenBackground, MHAND );
        DrawFatHand( hDC, oTime.hour*5+(oTime.minute/12),
                           hpenBackground, HHAND );
        DrawFatHand( hDC, oTime.minute=nTime.minute,
                           hpenForeground, MHAND );
        DrawFatHand( hDC, (oTime.hour=nTime.hour)*5+(oTime.minute/12),
                           hpenForeground, HHAND );
     }
   }

   if (!bIconic && nTime.second != oTime.second)
     /* Draw new second hand */
     DrawHand( hDC, oTime.second=nTime.second, hpenBackground, SECONDSCALE, R2_NOT );
  }
  GlobalUnlock( hcirTab );
}


static VOID PASCAL DrawFace(hDC)
  HDC hDC;
{
  int  i;
  RECT tRect;
  POINT FAR *pt;
  int  blobHeight, blobWidth;
  int  clockHeight, clockWidth;

  blobWidth = (MAXBLOBWIDTH * (long)(clockRect.right - clockRect.left)) / HorzRes;
  blobHeight = (blobWidth * aspectN) / aspectD;
  if (blobHeight < 2)
      blobHeight = 1;
  if (blobWidth < 2)
      blobWidth = 2;
  InflateRect( (LPRECT)&clockRect, -(blobHeight >> 1), -(blobWidth >> 1) );

  clockRadius = (clockRect.right - clockRect.left) >> 1;
  clockCenter.y = clockRect.top +
                     ((clockRect.bottom - clockRect.top) >> 1);
  clockCenter.x = clockRect.left + clockRadius;
  for (i=0; i<60; i++)
  {
    pt = lpcirTab + i;
    tRect.top = ((long)(pt->y) * clockRadius) / 8000 + clockCenter.y;
    tRect.left = ((long)(pt->x) * clockRadius) / 8000 + clockCenter.x;
    if (i % 5)
    {
       /*Draw a dot */
      if (blobWidth > 2 && blobHeight >= 2)
      {
        tRect.right = tRect.left + 1;
        tRect.bottom = tRect.top + 1;
        FillRect( hDC, (LPRECT)&tRect, hbrForeground );
      }
    }
    else
    {
      tRect.right = tRect.left + blobWidth;
      tRect.bottom = tRect.top + blobHeight;
      OffsetRect( (LPRECT)&tRect, -(blobWidth >> 1) , -(blobHeight >> 1) );
      FillRect( hDC, (LPRECT)&tRect, hbrForeground );
    }
  }
  InflateRect( (LPRECT)&clockRect, (blobHeight >> 1), (blobWidth >> 1) );
}


static VOID PASCAL DrawHand( hDC, pos, hPen, scale, patMode )
  HDC     hDC;
  int     pos;
  HPEN    hPen;
  int     scale;
  int     patMode;
{
  POINT   FAR *pt;
  int     radius;

#if defined(MEWEL_GUI) && (defined(__TURBOC__) || defined(MSC))
  if (patMode == R2_NOT)
    patMode = R2_XORPEN;
#endif

  MoveTo(hDC, clockCenter.x, clockCenter.y);
  radius = ((long) clockRadius * scale) / 100;
  pt = lpcirTab+pos;
  SetROP2(hDC, patMode);
  SelectObject(hDC, hPen);
  LineTo(hDC, clockCenter.x+(int)(((long)(pt->x)*(radius))/8000),
             clockCenter.y+(int)(((long)(pt->y)*(radius))/8000));
}


static VOID PASCAL DrawFatHand(hDC, pos, hPen, hHand)
  HDC     hDC;
  int     pos;
  HPEN    hPen;
  BOOL    hHand;
{
  POINT   *pt, tip, stip;
  int radius, scale;
  int n, m;

  SetROP2( hDC, 13 );
  SelectObject( hDC, hPen );
  scale = hHand ? 7 : 5;
  n = (pos+15)%60;
  m = (((long)clockRadius*scale) / 100);
  stip.y = (long)(lpcirTab[n].y) * m / 8000;

  stip.x = (long)(lpcirTab[n].x) * m / 8000;

  scale = hHand ? 65 : 80;
  tip.y = (long)(lpcirTab[pos].y)*(((long)clockRadius*scale)/100)/8000;
  tip.x = (long)(lpcirTab[pos].x)*(((long)clockRadius*scale)/100)/8000;

  MoveTo( hDC, clockCenter.x+stip.x, clockCenter.y+stip.y );
  LineTo( hDC, clockCenter.x+tip.x,  clockCenter.y+tip.y );
  MoveTo( hDC, clockCenter.x-stip.x, clockCenter.y-stip.y );
  LineTo( hDC, clockCenter.x+tip.x,  clockCenter.y+tip.y );

  scale = hHand ? 15 : 20;

  n = (pos+30)%60;
  m = (((long)clockRadius*scale) / 100);
  tip.y = (long)(lpcirTab[n].y) * m/ 8000;
  tip.x = (long)(lpcirTab[n].x) * m/ 8000;
  MoveTo( hDC, clockCenter.x+stip.x, clockCenter.y+stip.y );
  LineTo( hDC, clockCenter.x+tip.x,  clockCenter.y+tip.y );
  MoveTo( hDC, clockCenter.x-stip.x, clockCenter.y-stip.y );
  LineTo( hDC, clockCenter.x+tip.x,  clockCenter.y+tip.y );
}


static VOID PASCAL CompClockDim(void)
{
  int clockHeight, clockWidth;
  int tWidth, tHeight;

  tWidth = clockRect.right - clockRect.left;
  tHeight = clockRect.bottom - clockRect.top;
  if (tWidth > (tHeight * aspectD) / aspectN)
  {
    clockWidth = (tHeight * aspectD) / aspectN;
    clockRect.left += (tWidth-clockWidth) >> 1;
    clockRect.right = clockRect.left + clockWidth;
  }
  else
  {
    clockHeight = (tWidth * aspectN) / aspectD;
    clockRect.top += (tHeight-clockHeight) >> 1;
    clockRect.bottom = clockRect.top + clockHeight;
  }
}


BOOL FAR PASCAL ClockDemo(void)
{
  HWND    hWnd;
  TIME    nTime;
  char*   szTooMany;

  /*
    Register the clock class
  */
  if (!ClockInit(hAppInstance))
    return FALSE;

  ClockCreate((HWND) NULL);

  hWnd = CreateWindowEx(
        WS_EX_TOPMOST,
        "Clock",    /* The class name */
        "Clock",    /* The window instance name */
        (long) WS_CHILD | WS_THICKFRAME | WS_SYSMENU | WS_CAPTION | WS_MINIMIZEBOX | WS_CLIPSIBLINGS,
        CW_USEDEFAULT,
        0,                  /*y not used*/
	(HorzRes/3) + GetSystemMetrics(SM_CXFRAME)*2,
	(((HorzRes/3)*aspectN)/aspectD)+GetSystemMetrics(SM_CYCAPTION)+GetSystemMetrics(SM_CYFRAME)*2,
	hMain,       /* The window instance will be created with the class default menu */
        0,           /* null parent window parm*/
        hAppInstance,
        (LPSTR) NULL);

  GetTime((TIME *) &nTime);
  GetTime((TIME *) &oTime);
  while (nTime.second == oTime.second &&
         nTime.minute == oTime.minute &&
         nTime.hour   == oTime.hour)
    GetTime((TIME *) &oTime);

  if (!SetTimer(hWnd, TimerID, OPEN_TLEN, (long) 0))
  {
    /* Windows only supports 16 public timers */
    MessageBox(hMain, "Too many timers", "Clock",
               MB_OK | MB_ICONHAND | MB_SYSTEMMODAL);
    DeleteTools();
    return FALSE;
  }

  ShowWindow(hWnd, SW_SHOW);
  BringWindowToTop(hWnd);
  return TRUE;
}


static BOOL PASCAL ClockInit(hAppInstance)
  HANDLE    hAppInstance;
{
  HANDLE    hClockClass;
  PWNDCLASS pClockClass;
  HANDLE    hRes;
  char      szData[5];

  hClockClass = LocalAlloc(LPTR, sizeof(WNDCLASS));
  pClockClass = (PWNDCLASS) LocalLock(hClockClass);

  pClockClass->lpszClassName = (LPSTR) "Clock";
  pClockClass->hbrBackground = (HBRUSH)NULL;
  pClockClass->style         = CS_VREDRAW | CS_HREDRAW | CS_BYTEALIGNCLIENT;
  pClockClass->hInstance     = hAppInstance;
  pClockClass->lpfnWndProc   = ClockWndProc;
  pClockClass->hCursor       = LoadCursor(NULL, IDC_ARROW);
  pClockClass->hIcon         = NULL;

  if (!RegisterClass((LPWNDCLASS) pClockClass))
    return FALSE;

  LocalFree((HANDLE) hClockClass);

  /* Load in pre-computed cosine values from resource file */
  hRes = FindResource(hAppInstance, (LPSTR) "clock", (LPSTR) "data");
  if (hRes)
  {
    hcirTab = LoadResource(hAppInstance, hRes);
    LockResource(hcirTab);
  }
  else
    return FALSE;

  return TRUE;
}


#include <dos.h> 
#include <time.h> 

void GetTime(TIME *t)
{
#if defined(WC386)
  time_t now;
  struct tm *tm;

  time(&now);
  tm = localtime(&now);

  if ((t->hour = tm->tm_hour) > 12)
    t->hour -= 12;
  t->minute = tm->tm_min;
  t->second = tm->tm_sec;
#else
  union REGS r;

  r.x.ax = 0x2C00;
  intdos(&r, &r);

  if ((t->hour = r.h.ch) > 12)
    t->hour -= 12;
  t->minute = r.h.cl;
  t->second = r.h.dh;
#endif
}

