/*===========================================================================*/
/*                                                                           */
/* File    : HEXCALC.C                                                       */
/*                                                                           */
/* Purpose :                                                                 */
/*                                                                           */
/* History :                                                                 */
/*                                                                           */
/* (C) Copyright 1989-1992 Marc Adler/Magma Systems     All Rights Reserved  */
/*===========================================================================*/
#include "wmewldem.h"

DWORD CalcIt(DWORD dwFirstNum, WORD nOperation, DWORD dwNum);
VOID  ShowNumber (HWND hWnd, DWORD dwNumber);

extern HWND hWndCalc;


VOID FAR PASCAL Calculator(void)
{
  /*
    If the calculator is already in existence, just bring it to the top
  */
  if (hWndCalc)
  {
    BringWindowToTop(hWndCalc);
    SetFocus(hWndCalc);
    return;
  }

  /*
    Tell the user some info about the behavior of the calculator window
  */
  MessageBox(hMain,
    "The Calculator is a POPUP window, and will stay in front of all other \
non-popup windows (like the notepad).",
    "Note", MB_OK);

  /*
    Read the calc in from the rresource file
  */
  if ((hWndCalc = CreateDialog(hAppInstance,"HexCalc",0,(FARPROC)NULL)) != NULL)
  {
    SetFocus(hWndCalc);
  }
  else
  {
    MessageBox(hMain, "Could not load calculator", "Error", MB_OK);
  }
}


VOID ShowNumber(HWND hWnd, DWORD dwNumber)
{
   char szBuffer[20] ;
#ifdef MEWEL
   SetDlgItemText(hWnd, VK_ESCAPE, 
                    strupr(ltoa(dwNumber, szBuffer, 16))) ;
#endif
}


DWORD CalcIt(DWORD dwFirstNum, WORD nOperation, DWORD dwNum)
{
   switch (nOperation)
   {
      case '=' : return dwNum;
      case '+' : return dwFirstNum +  dwNum;
      case '-' : return dwFirstNum -  dwNum;
      case '*' : return dwFirstNum *  dwNum;
      case '&' : return dwFirstNum &  dwNum;
      case '|' : return dwFirstNum |  dwNum;
      case '^' : return dwFirstNum ^  dwNum;
      case '<' : return dwFirstNum << dwNum;
      case '>' : return dwFirstNum >> dwNum;
      case '/' : return dwNum ? dwFirstNum / dwNum : ULONG_MAX;
      case '%' : return dwNum ? dwFirstNum % dwNum : ULONG_MAX;
      default  : return 0L;
   }
}

LONG FAR PASCAL CalcDlgProc(HWND hWnd, WORD iMessage, WORD wParam, LONG lParam)
{
   static BOOL  bNewNumber = TRUE;
   static DWORD dwNumber, dwFirstNum;
   static short nOperation = '=';

   HWND         hButton;

   (void) lParam;

   switch (iMessage)
   {
     case WM_KEYDOWN:                   /* left arrow --> backspace */
       if (wParam != VK_LEFT)
         break;
       wParam = VK_BACK;
                                             /* fall through */
     case WM_CHAR:
       if ((wParam = toupper (wParam)) == VK_RETURN)
         wParam = '=';

       if ((hButton = GetDlgItem (hWnd, wParam)) != NULL)
       {
         SendMessage (hButton, BM_SETSTATE, 1, 0L);
         SendMessage (hButton, BM_SETSTATE, 0, 0L);
       }
       else
       {
         MessageBeep (0);
         break;
       }
                                             /* fall through */
    case WM_COMMAND:
      SetFocus (hWnd);

      if (wParam == VK_BACK)                  /* backspace */
        ShowNumber(hWnd, dwNumber /= 16);
      else if (wParam == VK_ESCAPE)           /* escape */
        ShowNumber(hWnd, dwNumber = 0L);
      else if (isxdigit(wParam))              /* hex digit */
      {
        if (bNewNumber)
        {
          dwFirstNum = dwNumber;
          dwNumber = 0L;
        }
        bNewNumber = FALSE;

        if (dwNumber <= (ULONG_MAX >> 4))
          ShowNumber(hWnd, dwNumber = 16 * dwNumber + wParam -
                     (isdigit (wParam) ? '0' : 'A' - 10));
        else
          MessageBeep(0);
     }
     else                                    /* operation */
     {
       if (!bNewNumber)
         ShowNumber(hWnd, dwNumber = CalcIt(dwFirstNum, nOperation, dwNumber));
       bNewNumber = TRUE;
       nOperation = wParam;
     }
     break;


    case WM_DESTROY:
      hWndCalc = NULL;
      break;

    default :
       return DefWindowProc(hWnd, iMessage, wParam, lParam);
  }

  return TRUE;
}

