/*===========================================================================*/
/*                                                                           */
/* File    : LISTBOX.C                                                       */
/*                                                                           */
/* Purpose : Demo of a dialog with a listbox and radiobuttons.               */
/*                                                                           */
/* History :                                                                 */
/*                                                                           */
/* (C) Copyright 1989-1992 Marc Adler/Magma Systems     All Rights Reserved  */
/*===========================================================================*/
#include "wmewldem.h"

#ifdef MODELESSTEST
extern HWND hWndListbox1;
extern HWND hWndListbox2;
extern HWND hWndString;
#endif


VOID FAR PASCAL ListboxDemo(void)
{
  FARPROC lpfn;
#ifdef MODELESSTEST
  HWND    *pHwnd;
#endif

  if (AreTimersRunning())
    KillAllTimers();

  lpfn = MakeProcInstance((FARPROC) ListboxDialogProc, hAppInstance);
#ifdef MODELESSTEST
  pHwnd = (hWndListbox1 != NULL) ? &hWndListbox2 : &hWndListbox1;
  if ((*pHwnd = CreateDialog(hAppInstance,"ListDemoDlg",hMain,lpfn)) != NULL)
  {
    ShowWindow(*pHwnd, SW_SHOW);
    SetFocus(*pHwnd);
    hWndString = GetDlgItem(*pHwnd, ID_STRING);
  }
#else
  DialogBox(hAppInstance, "ListDemoDlg", hMain, lpfn);
  FreeProcInstance(lpfn);
#endif
}


int FAR PASCAL ListboxDialogProc(HWND hDlg, WORD message, WORD wParam, DWORD lParam)
{
  char szString[130];
  int  iPos;
  int  operation;
  int  iSel;

#ifdef MODELESSTEST
  static HBRUSH hBlueBrush = 0;

  if (!hBlueBrush)
    hBlueBrush = CreateSolidBrush(RGB(0, 0, 255));
#endif

  (void) lParam;

  switch (message)
  {
    case WM_INITDIALOG :
      CheckRadioButton(hDlg, ID_APPEND, ID_DELETE, ID_APPEND);

#define READFROMFILE
#ifdef READFROMFILE
      {
      FILE *f;
      if ((f = fopen("foo", "r")) != NULL)
      {
        while (fgets(szString, sizeof(szString), f))
        {
          szString[strlen(szString)-1] = '\0';  /* get rid of newline */
          SendDlgItemMessage(hDlg, ID_LISTBOX, LB_ADDSTRING, 0, 
                             (LONG) (LPSTR) szString);
        }
        fclose(f);
      }
      }
#endif

      return TRUE;


    case WM_COMMAND    :
      switch (wParam)
      {
/*#define DELTA*/
#ifdef DELTA
        /*
          Test of auto-tabbing with WM_NEXTDLGCTL.
          Tab to the next field after 3 characters are entered.
        */
        case ID_STRING :
          if (HIWORD(lParam) == EN_UPDATE)
          {
            int n = GetDlgItemText(hDlg,ID_STRING,szString,sizeof(szString));
            if (n >= 3)
              PostMessage(hDlg, WM_NEXTDLGCTL, 0, 0L);
          }
          break;
#endif

        case ID_INSERT :
        case ID_DELETE :
        case ID_APPEND :
          CheckRadioButton(hDlg, ID_APPEND, ID_DELETE, wParam);
          break;

      	case IDOK      :
          /*
            The user pressed the OK button. We want to get the string
            that is to be inserted as well as the position.
          */
          GetDlgItemText(hDlg, ID_STRING, (LPSTR) szString, sizeof(szString));
          iPos = GetDlgItemInt(hDlg, ID_POS, NULL, TRUE);

          /*
            Determine the operation to perform
          */
          operation = 0;
          if (IsDlgButtonChecked(hDlg, ID_APPEND))
            operation = LB_ADDSTRING;
          else if (IsDlgButtonChecked(hDlg, ID_INSERT))
            operation = LB_INSERTSTRING;
          else if (IsDlgButtonChecked(hDlg, ID_DELETE))
            operation = LB_DELETESTRING;

          if (operation)
          {
            SendDlgItemMessage(hDlg, ID_LISTBOX, operation, iPos,
                                      (DWORD) (LPSTR) szString);
            MessageBox(hMain, szString, "Completed", MB_OK | MB_ICONEXCLAMATION);
          }
          break;

        case IDCANCEL  :
          EndDialog(hDlg, 0);
          break;
      }

      return TRUE;


#ifdef MODELESSTEST
    case WM_SETFOCUS :
      SetFocus(GetDlgItem(hDlg, ID_STRING));
      return TRUE;

    case WM_CTLCOLOR :
    if (HIWORD(lParam) == CTLCOLOR_EDIT)
    {
      HBRUSH hBrush = hBlueBrush;
      SetTextColor(wParam, RGB(0, 255, 255));   // cyan fore
      SetBkColor(wParam, RGB(0, 0, 255));       // blue background
      return hBrush;
    }
    else
      return GetStockObject(LTGRAY_BRUSH);


    case WM_CLOSE :
      if (hDlg == hWndListbox1)
	hWndListbox1 = NULL;
      else
	hWndListbox2 = NULL;
      DestroyWindow(hDlg);
      return TRUE;
#endif

  }

  return FALSE;
}

