/*===========================================================================*/
/*                                                                           */
/* File    : NOTEPAD.C                                                       */
/*                                                                           */
/* Purpose : Demo of a multi-line edit control.                              */
/*                                                                           */
/*===========================================================================*/
#include "wmewldem.h"

static HWND  hWndNotepad, hEdit;

LONG FAR PASCAL NotepadWndProc(HWND,WORD,WORD,LONG);


VOID FAR PASCAL NotePad(VOID)
{
  RECT r;

  /*
    If the notepad is already in existence, just bring it to the top.
  */
  if (hWndNotepad)
  {
    BringWindowToTop(hWndNotepad);
    SetFocus(hWndNotepad);
    return;
  }

  /*
    Create the notepad frame
  */
  hWndNotepad = CreateWindow("NotePad",    /* class */
                             "NotePad",    /* title */
                             WS_BORDER | WS_CAPTION | WS_SYSMENU |
                             WS_THICKFRAME | WS_CHILD,
                             MAKE_X(5),  MAKE_Y(0),
                             MAKE_X(60), MAKE_Y(20),
                             hMain,          /* hParent */
                             NULL,           /* hMenu */
                             hAppInstance,   /* hInst */
                             0L);            /* lpParam */

  GetClientRect(hWndNotepad, (LPRECT) &r);

  /*
    Create the edit control which goes inside the notepad frame
  */
  hEdit = CreateWindow("Edit",         /* class */
                       NULL,           /* title */
                       WS_VSCROLL |
                       ES_MULTILINE | WS_CHILD | WS_VISIBLE,  /* dwStyle */
                       0,0,
                       CX_RECT(r), CY_RECT(r),
                       hWndNotepad,    /* hParent */
                       0,              /* hMenu */
                       hAppInstance,   /* hInst */
                       0L);            /* lpParam */

  /*
    Display the notepad and set the focus to the edit control
  */
  ShowWindow(hWndNotepad, SW_SHOW);
  UpdateWindow(hWndNotepad);
  BringWindowToTop(hWndNotepad);
  SetFocus(hEdit);
}


LONG FAR PASCAL NotepadWndProc(HWND hWnd, WORD message, WORD wParam, LONG lParam)
{
  char    buf[128];
  HANDLE  pHandle;

  switch (message)
  {
    case WM_COMMAND    :
      switch (wParam)
      {
        case ID_EXIT:
          PostMessage(hWnd, WM_CLOSE, 0, 0L);
          break;

        case ID_CUT :
          SendMessage(hEdit, WM_CUT, 0, 0L);
          break;
        case ID_CLEAR:
          SendMessage(hEdit, WM_CLEAR, 0, 0L);
          break;
        case ID_COPY:
          SendMessage(hEdit, WM_COPY, 0, 0L);
          break;
        case ID_PASTE:
          SendMessage(hEdit, WM_PASTE, 0, 0L);
          break;

        case ID_GETLEN :
          wsprintf(buf, "There are %d characters", 
                  (int) SendMessage(hEdit, WM_GETTEXTLENGTH, 0, 0L));
          MessageBox(hMain, buf, "Text Length", MB_OK);
          break;
        case ID_GETLINES:
          wsprintf(buf, "There are %d lines", 
                  (int) SendMessage(hEdit, EM_GETLINECOUNT, 0, 0L));
          MessageBox(hMain, buf, "# of Lines", MB_OK);
          break;
        case ID_GETHANDLE:
          pHandle = (HANDLE) SendMessage(hEdit, EM_GETHANDLE, 0, 0L);
          wsprintf(buf, "The edit handle is [%x]", pHandle);
          MessageBox(hMain, buf, "Get Handle", MB_OK);
          break;
      }
      break;

    case WM_SIZE  :
      if (hEdit)
        MoveWindow(hEdit, 0, 0, LOWORD(lParam), HIWORD(lParam), TRUE);
      break;

    case WM_CLOSE :
      hWndNotepad = NULL;
      DestroyWindow(hWnd);
      return TRUE;

    default :
      /* Call the default window procedure for the main window */
      return DefWindowProc(hWnd, message, wParam, lParam);
  }
  return TRUE;
}

