/*===========================================================================*/
/*                                                                           */
/* File    : SCROLBAR.C                                                      */
/*                                                                           */
/* Purpose : Demos the usage of scrollbars and scrollbar tracking.           */
/*                                                                           */
/* History :                                                                 */
/*                                                                           */
/* (C) Copyright 1989-1992 Marc Adler/Magma Systems     All Rights Reserved  */
/*===========================================================================*/
#include "wmewldem.h"


VOID FAR PASCAL PrintScrollbarValues(HWND hWnd,WORD message,WORD wParam,LONG lParam)
{
  char   buf[80];
  char   *pszMsg;
  int    nCode;
  int    minpos, maxpos, currpos;
  HDC    hDC;
  HBRUSH hBrush;
  RECT   r;

  (void) lParam;

  /*
    Get the scrollbar position and range. Don't print the message if
    the position is the same as it was in the last message.
  */
  nCode = (message == WM_HSCROLL) ? SB_HORZ : SB_VERT;
  if (wParam == SB_THUMBTRACK || wParam == SB_THUMBPOSITION)
  {
    if ((currpos = LOWORD(lParam)) == GetScrollPos(hWnd, nCode))
      return;
  }
  else
    currpos = GetScrollPos(hWnd, nCode);
  GetScrollRange(hWnd, nCode, &minpos, &maxpos);


  switch (wParam)
  {
    case SB_LINEUP   :
      SetScrollPos(hWnd, nCode, max(currpos-1, minpos), TRUE);
      currpos = GetScrollPos(hWnd, nCode);
      pszMsg = "SB_LINEUP";
      break;

    case SB_LINEDOWN :
      SetScrollPos(hWnd, nCode, min(currpos+1, maxpos), TRUE);
      currpos = GetScrollPos(hWnd, nCode);
      pszMsg = "SB_LINEDOWN";
      break;

    case SB_PAGEUP   :
      SetScrollPos(hWnd, nCode, max(currpos-10, minpos), TRUE);
      currpos = GetScrollPos(hWnd, nCode);
      pszMsg = "SB_PAGEUP";
      break;

    case SB_PAGEDOWN :
      SetScrollPos(hWnd, nCode, min(currpos+10, maxpos), TRUE);
      currpos = GetScrollPos(hWnd, nCode);
      pszMsg = "SB_PAGEDOWN";
      break;

    case SB_THUMBTRACK :
      pszMsg = "SB_THUMBTRACK";
      break;
    case SB_THUMBPOSITION :
      pszMsg = "SB_THUMBPOSITION";
      SetScrollPos(hWnd, nCode, currpos, TRUE);
      break;
    case SB_TOP        :
      pszMsg = "SB_TOP";
      goto do_it;
    case SB_BOTTOM     :
      pszMsg = "SB_BOTTOM";
      goto do_it;
    case SB_ENDSCROLL  :
      pszMsg = "SB_ENDSCROLL";
do_it:
      currpos = GetScrollPos(hWnd, nCode);
      SetScrollPos(hWnd, nCode, currpos, TRUE);
      break;
  }

  /*
    Format the scrollbar output message
  */
  wsprintf(buf, "Msg [%s]   Pos [%3d]  %3d%%     ",
                       (LPSTR) pszMsg, currpos, currpos * 100 / maxpos);


  /*
    Let's test out setting foreground and background colors for our
    TextOut call. Use a cyan background and blue text.
  */
  hDC = GetDC(hMsgWnd);

  hBrush = CreateSolidBrush(RGB(0, 0x80, 0x80));
  GetClientRect(hMsgWnd, (LPRECT) &r);
  FillRect(hDC, (LPRECT) &r, hBrush);
  DeleteObject(hBrush);

  SetTextColor(hDC, RGB(0, 0, 0x80));
  SetBkColor(hDC, RGB(0, 0x80, 0x80));
  TextOut(hDC, 0, 0, (LPSTR) buf, lstrlen(buf));

  ReleaseDC(hMsgWnd, hDC);
}

